#ifndef _ADDRESSER_H
#define _ADDRESSER_H

#include <QObject>


// A general interface for different engines' “addressing” (geocoding) mechanisms.
class Addresser : public QObject {
    Q_OBJECT

public:
    // The method to be implemented by engine-specific subclasses: This starts a request
    // for geocoding, to be responded to via the below signals.
    virtual void locateAddress(QString address) = 0;

signals:
    // Emitted when the address has been successfuly translated into a coordinate, passing
    // the found coordinate and engine-provided address label. Must be sent by subclasses.
    void locationFound(std::vector<double> point, QString name);

    // Emitted when the address could not be geocoded, for a given reason.
    void locationError(QString message);

    // Emitted when the geocoding has been cancelled for some reason (e.g., user-input
    // from an AddressSelectionWindow). If an addresser will never emit locationFound for
    // a given request, it *must* emit either this or locationError.
    void locationCancelled();
};


#endif // _ADDRESSER_H
