#include "addressselectionwindow.h"
#include "./ui_addressselectionwindow.h"


// A dialog that allows the user to select a location among a list of coordinates and
// address-labels. Its most notable signals are AddressSelectionWindow::addressSelected
// and QDialog::refused, which correspond to the user's location selection.
AddressSelectionWindow::AddressSelectionWindow(QWidget *parent, vector_2d_double coords, QStringList names)
    : QDialog(parent),
      ui(new Ui::AddressDialog)
{
    points = coords;
    pointNames = names;

    setupUi();
    populateLocationList();
}


AddressSelectionWindow::~AddressSelectionWindow()
{
    delete ui;
}


// An inherited slot from QDialog; this is called when the user
// hits the “done” button.
void
AddressSelectionWindow::accept()
{
    int index = ui->locationList->currentRow();
    if (index >= 0)
	emit addressSelected(points[index], pointNames[index]);

    QDialog::accept();
}


// When a location on the list is “activated” (double-click/enter), report
// it as selected and close the window.
// Handles signal QListWidget::itemActived (locationList).
void
AddressSelectionWindow::locationActivated(QListWidgetItem* item)
{
    int index = ui->locationList->row(item);
    emit addressSelected(points[index], pointNames[index]);

    QDialog::accept();
}


// When a location on the list is selected, enable/disable the accept button.
// Handles signal QListWidget::itemSelectionChanged (locationList).
void
AddressSelectionWindow::locationSelected()
{
    QDialogButtonBox* box = ui->buttonBox;
    if (ui->locationList->currentRow() >= 0
	&& !(box->standardButtons() & QDialogButtonBox::Ok))
    {
	ui->buttonBox->addButton(QDialogButtonBox::Ok);
    }
}


// Populate the coordinate/address list with selectable options.
void
AddressSelectionWindow::populateLocationList()
{
    for (int i = 0; i < points.size(); i++)
	ui->locationList->addItem(pointNames[i]);
}


// Initialize the UI; set default values; connect signals, etc.
void
AddressSelectionWindow::setupUi()
{
    ui->setupUi(this);

    // Handle list events…
    connect(ui->locationList, &QListWidget::itemActivated,
	    this, &AddressSelectionWindow::locationActivated);
    connect(ui->locationList, &QListWidget::itemSelectionChanged,
	    this, &AddressSelectionWindow::locationSelected);
}
