#ifndef _MAINWINDOW_H
#define _MAINWINDOW_H

#include <QMainWindow>
#include <QStyle>

#include "routingkit.h"
#include "util.h"

class QAbstractButton;

class Addresser;
class FOSSGISMapData;
class MapData;
class OpenRouteServiceMapData;
class Router;


#define ORS_API_KEY "5b3ce3597851110001cf62481fcc844a640c4894a09a3f64f2abacd7"


QT_BEGIN_NAMESPACE
namespace Ui { class MainWindow; }
QT_END_NAMESPACE


// MainWindow — The primary window used by the application.
class MainWindow : public QMainWindow
{
    Q_OBJECT


public:
    MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

    void calculateDistances();
    void beginCalculateDistances();
    void displayDistances(vector_2d_double points, vector_2d_int distances);

    // Signal handlers relating to distance calculations.
    void calculationError(QString text);
    void calculationProgress(float percent, float percentIncrement,  QString text);
    void distancesCalculated(vector_2d_double points, vector_2d_int distances);

    // Signal handlers relating to parsing of map-data.
    void mapDataError(QString text);
    void mapDataProgress(float percent, float percentIncrement, QString text);
    void mapDataFinished();

    // Loading map-data.
    void loadRkMapData();
    void loadOrsMapData();
    void loadOsrmMapData();
    void connectMapData(MapData* mapdata);

    // Menubar signal-handlers.
    void openAboutWindow();
    void copyCSVData();
    void clearLocations();
    void repopulateTableAndCSV(bool checked);

    // Location management signal-handlers.
    void addLocationString();
    void addLocationAddressError(QString message);
    void addLocationAddressCancelled();
    void addLocationCoordinates(std::vector<double> point, QString name);
    void removeSelectedLocation();

    // Engine-specific input signal-handlers.
    void rkBrowseMapFile();
    void rkFileChanged(const QString& text);
    void orsApiKeyToggled(int state);
    void orsApiOrServerChanged(const QString& text);
    void orsGeocodingToggled(int state);
    void osrmServerChanged(const QString& text);

    // Misc input-validation signal-handlers (primarily disables/enables widgets based on input).
    void locationAddressChanged(const QString& text);
    void engineSelected(int tabIndex);
    void locationSelected(int row);

private:
    bool validateOverall();

    QString currentEngine();
    bool travelByCar();

    void addToLocationsList(std::vector<double> point, QString name);

    void populateTableWithDistances(vector_2d_double points, vector_2d_int distances);
    void populateTextBrowserWithDistances(vector_2d_double points, vector_2d_int distances);

    vector_2d_double coordListTo2dVector();

    void resetAddresser();
    void setupUi();

    Ui::MainWindow* ui;

    OpenRouteServiceMapData* orsMapData;
    FOSSGISMapData* osrmMapData;
    RoutingKitMapData<RKDrivingGraph>* rkMapDataDriving;
    RoutingKitMapData<RKWalkingGraph>* rkMapDataWalking;

    Addresser* addresser;
    Router* currentRouter;
    bool calculating;

    vector_2d_double previousPoints;
    vector_2d_int previousDistances;
    QStringList previousAddresses;
};



void use_button_icon_or_text(QAbstractButton* button);
void set_button_qt_icon(QAbstractButton* button, QStyle::StandardPixmap icon);


#endif // _MAINWINDOW_H
