#ifndef _MAPDATA_H
#define _MAPDATA_H

#include <QObject>


// An object representing an engine's mapping data; this will be passed to a
// corresponding Router or Addresser for use in calculations.
class MapData : public QObject {
    Q_OBJECT

public:
    MapData(bool by_car = true) { byCar = by_car; }

    // Data will often be passed to the MapData object by it's particular
    // subclass'es constructor; if that data requires any special processing,
    // this method should be overloaded by that processing (e.g., map-file
    // parsing). This method *must* emit either MapData::mapDataError or
    // MapData::mapDataFinished.
    virtual bool loadMap() {
	emit MapData::mapDataFinished();
	return true;
    }

    bool byCar;

signals:
    // Signals that the map-data has been loaded, and that this MapData is ready
    // to be used by its engine's Router.
    void mapDataFinished();

    // Signals an error in loading the data, along with a user-facing error message.
    void mapDataError(QString text);

    // Signals progress in loading map-data to the user, through the progress bar; if
    // percentIncrement is above zero, the progress is incremented by that amount,
    // until 100%. If totalPercent is specified and between 0 and 100, the progress
    // is set directly to that amount. The description details the current task being
    // undertaken.
    void mapDataProgress(float totalPercent, float percentIncrement, QString description);
};


#endif // _MAPDATA_H
