#include "netaddresser.h"

#include "downloader.h"


// A class for all Addressers that use HTTP APIs, adding a generic interface for
// downloading and parsing the relevant data.
NetAddresser::NetAddresser()
{
    downloader = new Downloader();
}


NetAddresser::~NetAddresser()
{
    delete downloader;
}


// The template for downloading/parsing to be used by all NetAddressers.
// The invoked methods `parseCoordinatesFromResponse` and `makeLocatingRequest`
// are how any engine subclasses actually implement their geocoding.
void
NetAddresser::locateAddress(QString address)
{
    connect(downloader, &Downloader::downloadError,
	    this, &NetAddresser::downloadError);
    connect(downloader, &Downloader::downloadFinished,
	    this, &NetAddresser::parseCoordinatesFromResponse);

    downloader->startDownload(makeLocatingRequest(address), makePostData(address));
}


// Emit a downloader error, as necessary.
// Handles signal Downloader::downloadError (downloader).
void
NetAddresser::downloadError(QString message)
{
    emit Addresser::locationError("Připojování k serveru…");
}
