#ifndef _OPENROUTESERVICE_H
#define _OPENROUTESERVICE_H

#include "mapdata.h"
#include "netaddresser.h"
#include "netrouter.h"

class QNetworkRequest;

// All classes relating to the OpenRouteService engine.


// OpenRouteService map-data container. In this case, it stores only the URL & API key.
class OpenRouteServiceMapData : public MapData {
    Q_OBJECT

public:
    OpenRouteServiceMapData(QString server_url, QString api_key)
    {
	serverUrl = server_url;
	apiKey = api_key;
    }

    QString serverUrl;
    QString apiKey;
};



// Distance-calculation using OpenRouteService's HTTP API.
class OpenRouteServiceRouter : public NetRouter {
    Q_OBJECT

public:
    OpenRouteServiceRouter(OpenRouteServiceMapData* map_data);

protected:
    virtual QNetworkRequest* makeRoutingRequest(vector_2d_double points);
    virtual QByteArray* makePostData(vector_2d_double points);

private:
    OpenRouteServiceMapData* mapData;
};



// “Addressing” (geocoding) using OpenRouteService's HTTP API.
class OpenRouteServiceAddresser : public NetAddresser {
    Q_OBJECT

public:
    OpenRouteServiceAddresser(OpenRouteServiceMapData* map_data);

    virtual void parseCoordinatesFromResponse(QByteArray* data);
    virtual QNetworkRequest* makeLocatingRequest(QString address);

    void addressSelected(std::vector<double> point, QString name);
    void noAddressSelected();

private:
    OpenRouteServiceMapData* mapData;
};


#endif // _OPENROUTESERVICE_H
