#ifndef _ROUTER_H
#define _ROUTER_H

#include <QObject>

#include "util.h"


// A general interface for different engines' routing (distance-calculation) mechanisms.
class Router : public QObject {
    Q_OBJECT

public:
    // Must be implemented by engine subclasses: This returns the distance between
    // two points in meters, and is invoked by this class'es `calculateDistances`
    // method.
    virtual int pointDistance(double from_latitude, double from_longitude,
			      double to_latitude, double to_longitude) = 0;

    // A loop, calling `pointDistance` over and over until the distance matrix is
    // complete. If re-implemented, `pointDistance` might be unnecessary to define,
    // though the signals Router::distancesCalculated and router::calculationError
    // *must* be emitted at some point.
    virtual void calculateDistances(vector_2d_double points);

signals:
    // Signal, communicating the results of the distance calculation.
    void distancesCalculated(vector_2d_double points, vector_2d_int distances);

    // Signals an error in calculation, along with a user-facing error message.
    void calculationError(QString text);

    // Signals progress in the calculation to the user, through the progress bar; if
    // percentIncrement is above zero, the progress is incremented by that amount,
    // until 100%. If totalPercent is specified and between 0 and 100, the progress
    // is set directly to that amount. The description details the current task being
    // undertaken.
    void calculationProgress(float totalPercent, float percentIncrement, QString description);
};


#endif // _ROUTER_H
